<?php

declare(strict_types=1);

namespace Drupal\lms_certificate_fillpdf\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Controller\ControllerResolverInterface;
use Drupal\fillpdf\Service\FillPdfLinkManipulator;
use Drupal\group\Entity\GroupInterface;
use Drupal\lms\Entity\CourseStatusInterface;
use Drupal\lms\TrainingManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Handles certificates for courses.
 *
 * @todo move this logic to a controller in the main lms_certificate module, and
 * the fillpdf-specific logic to a plugin or tagged service implementation
 * called by that controller.
 * @see https://www.drupal.org/project/lms_certificate/issues/3502791
 */
class CertificateController extends ControllerBase {

  public function __construct(
    protected ControllerResolverInterface $controllerResolver,
    protected TrainingManager $trainingManager,
    protected RequestStack $requestStack,
    protected FillPdfLinkManipulator $linkManipulator,
    protected HttpKernelInterface $httpKernel,
  ) {}

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('controller_resolver'),
      $container->get('lms.training_manager'),
      $container->get('request_stack'),
      $container->get('fillpdf.link_manipulator'),
      $container->get('http_kernel'),
    );
  }

  /**
   * Generates a certificate given a group.
   */
  public function getCertificate(GroupInterface $group): Response {
    // Re-use the fillpdf controller to generate the merged PDF form
    // certificate. The controller relies on query arguments from the current
    // URL so call it via a subrequest.
    // @todo if fill PDF adds an API to generate a PDF from context, use that
    // instead.
    $fid = $group->get('certificate')->entity->id();
    $course_status = $this->trainingManager->loadCourseStatus($group, $this->currentUser(), ['status' => CourseStatusInterface::STATUS_PASSED]);
    $query = [
      'fid' => $fid,
      'entity_ids' => [
        'user' => [$this->currentUser()->id()],
        'group' => [$group->id()],
        'lms_course_status' => [$course_status->id()],
      ],
    ];

    $link = $this->linkManipulator->generateLink($query);
    $sub_request = Request::create($link->toString());
    $response = $this->httpKernel->handle($sub_request, HttpKernelInterface::SUB_REQUEST);
    return $response;
  }

  /**
   * Access callback.
   */
  public function access(GroupInterface $group): AccessResult {
    $access_result = AccessResult::allowedIf(!$group->get('certificate')->isEmpty())->addCacheableDependency($group);
    $course_status = $this->trainingManager->loadCourseStatus($group, $this->currentUser(), ['status' => CourseStatusInterface::STATUS_PASSED]);
    $access_result->andIf(AccessResult::allowedIf($course_status));

    if ($course_status) {
      // If the user has passed, cache the result - the situation can only
      // change if the course status changes again to a non-passed state.
      $access_result->addCacheableDependency($course_status);
    }
    else {
      $access_result->setCacheMaxAge(0);
    }
    // Set to cache per user even though the resulting response may be
    // uncacheable.
    $access_result->cachePerUser();
    return $access_result;
  }

}
